/*
 *  Copyright (c) 2022, WSO2 LLC. (http://www.wso2.org) All Rights Reserved.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */

package model

import (
	"testing"

	"github.com/stretchr/testify/assert"
)

func TestGetAuthorityHeader(t *testing.T) {
	type getXWso2AuthorityHeaderTestItem struct {
		serviceURL      string
		authorityHeader string
		message         string
	}
	dataItems := []getXWso2AuthorityHeaderTestItem{
		{
			serviceURL:      "https://interceptor-service-host:3000",
			authorityHeader: "interceptor-service-host:3000",
			message:         "invalid authority header when port is present",
		},
		{
			serviceURL:      "https://abcd.wxyz.amazonaws.com",
			authorityHeader: "abcd.wxyz.amazonaws.com:443",
			message:         "invalid authority header when port is implicit",
		},
	}
	for _, item := range dataItems {
		endpoint, _ := getHTTPEndpoint(item.serviceURL)
		authHeader := endpoint.GetAuthorityHeader()
		assert.Equal(t, authHeader, item.authorityHeader, item.message)
	}
}
