/*
 *  Copyright (c) 2021, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */

package org.wso2.apk.enforcer.commons.analytics.collectors;

import org.wso2.apk.enforcer.commons.analytics.exceptions.DataNotFoundException;
import org.wso2.apk.enforcer.commons.analytics.publishers.dto.API;
import org.wso2.apk.enforcer.commons.analytics.publishers.dto.Application;
import org.wso2.apk.enforcer.commons.analytics.publishers.dto.Error;
import org.wso2.apk.enforcer.commons.analytics.publishers.dto.Latencies;
import org.wso2.apk.enforcer.commons.analytics.publishers.dto.MetaInfo;
import org.wso2.apk.enforcer.commons.analytics.publishers.dto.Operation;
import org.wso2.apk.enforcer.commons.analytics.publishers.dto.Target;
import org.wso2.apk.enforcer.commons.analytics.publishers.dto.enums.EventCategory;
import org.wso2.apk.enforcer.commons.analytics.publishers.dto.enums.FaultCategory;

import java.util.Collections;
import java.util.Map;

/**
 * Data provider interface to extract request data.
 */
public interface AnalyticsDataProvider {

    EventCategory getEventCategory();

    boolean isAnonymous();

    boolean isAuthenticated();

    FaultCategory getFaultType();

    API getApi() throws DataNotFoundException;

    Application getApplication() throws DataNotFoundException;

    Operation getOperation() throws DataNotFoundException;

    Target getTarget();

    Latencies getLatencies();

    MetaInfo getMetaInfo();

    int getProxyResponseCode();

    int getTargetResponseCode();

    long getRequestTime();

    Error getError(FaultCategory faultCategory);

    String getUserAgentHeader();

    String getUserName();

    String getEndUserIP();

    default Map<String, Object> getProperties() {
        return Collections.EMPTY_MAP;
    }
}
